import 'dart:collection';

import 'package:flutter/material.dart';
import 'package:select_dialog/select_dialog.dart';
import 'dart:async';
import 'dart:developer';

import 'package:flutter/services.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:intl/intl.dart';
import 'package:smile_flutter/smile_flutter.dart';
import 'dart:io' show Platform;

void main() {
  runApp(MyApp());
}

class MyApp extends StatefulWidget {
  @override
  _MyAppState createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: AppPage(),
      builder: EasyLoading.init(),
    );
  }
}

class DropDownType {
  String name = "";
  String value = "";

  DropDownType(String name, String value) {
    this.name = name;
    this.value = value;
  }

  bool operator ==(dynamic other) =>
      other != null && other is DropDownType && this.name == other.name;

  @override
  int get hashCode => super.hashCode;
}

class AppPage extends StatefulWidget {
  // Platform messages are asynchronous, so we initialize in an async method.
  @override
  _AppPageState createState() => _AppPageState();
}

class _AppPageState extends State<AppPage> {
  var currentUserId = null;
  var isProduction = false;
  var countryDropDownValue = new DropDownType('Select Country', "select");
  var idTypeDropDownValue = new DropDownType('Select Id Type', "select");

  final idNumberController = TextEditingController();

  final firstNameController = TextEditingController();

  final middleNameController = TextEditingController();

  final lastNameController = TextEditingController();

  AlertDialog showAlert(BuildContext context, String title, String body) {
    Widget okButton = TextButton(
      child: Text("OK"),
      onPressed: () {
        Navigator.of(context).pop();
      },
    );

    return AlertDialog(
      title: Text(title),
      content: Text(body),
      actions: [
        okButton,
      ],
    );
  }

  handleSelfieResult(BuildContext context, Map<dynamic, dynamic>? result) {
    var resultCode = result!["SID_RESULT_CODE"];
    var resultTag = result["SID_RESULT_TAG"];
    var title = "Selfie Capture Failed";
    var body =
        "Failed selfie capture with error ${resultCode} tag ${resultTag}";
    if (resultCode == -1) {
      title = "Selfie Capture Success";
      body =
          "Successfully captured selfie with tag ${result["SID_RESULT_TAG"]}";
    }
    showDialog(
      context: context,
      builder: (BuildContext context) {
        return showAlert(context, title, body);
      },
    );
  }

  processResponse(Map<dynamic, dynamic>? response) {
    countryDropDownValue = new DropDownType('Select Country', "select");
    idTypeDropDownValue = new DropDownType('Select Id Type', "select");
    idNumberController.text = "";
    firstNameController.text = "";
    middleNameController.text = "";
    lastNameController.text = "";
    if (response != null) {
      if (response["result"]["ResultCode"] != null &&
          response["result"]["ResultText"] != null) {
        setState(() {
          currentUserId = response["result"]["PartnerParams"]["user_id"];
        });
        EasyLoading.showSuccess(
            "Job Submission Succeededwith result code ${response["result"]["ResultCode"]} and result text ${response["result"]["ResultText"]}");
        return;
      }
    }
    EasyLoading.showError("Oops something went wrong, please try again");
  }

  doEnrollWithIDCard() async {
    var result = await SmileFlutter.captureSelfieAndIDCard("") ?? null;
    var resultCode = result!["SID_RESULT_CODE"];
    var resultTag = result["SID_RESULT_TAG"];
    if (resultCode == -1) {
      try {
        EasyLoading.show(status: 'loading...');
        var submitResult = await SmileFlutter.submitJob(
            resultTag, 1, isProduction, "https:test.com", null, null, null);
        EasyLoading.dismiss();
        processResponse(submitResult);
        return;
      } catch (e) {
        EasyLoading.showError("Oops something went wrong");
      }
      return;
    }
  }

  doEnrollWithIDNumber(BuildContext context) async {
    var result = await SmileFlutter.captureSelfie("") ?? null;
    var resultCode = result!["SID_RESULT_CODE"];
    var resultTag = result["SID_RESULT_TAG"];
    if (resultCode == -1) {
      showIdInfoDialog(context, 1, tag: resultTag);
      return;
    }
    EasyLoading.showError("Oops something went wrong");
  }

  doEnroll({String tag = ""}) async {
    var result = await SmileFlutter.captureSelfie(tag) ?? null;
    var resultCode = result!["SID_RESULT_CODE"];
    var resultTag = result["SID_RESULT_TAG"];
    if (resultCode == -1) {
      try {
        EasyLoading.show(status: 'loading...');
        var submitResult = await SmileFlutter.submitJob(
            resultTag, 4, isProduction, "https:test.com", null, null, null);
        EasyLoading.dismiss();
        processResponse(submitResult);
        return;
      } catch (e) {
        print(e);
        EasyLoading.showError("Oops something went wrong");
      }
      return;
    }
  }

  doDocVerificaton({String tag = ""}) async {
    var result = await SmileFlutter.captureSelfieAndIDCard(tag) ?? null;
    var resultCode = result!["SID_RESULT_CODE"];
    var resultTag = result["SID_RESULT_TAG"];
    if (resultCode == -1) {
      try {
        showIdInfoDialog(context, 6, tag: resultTag);
        return;
      } catch (e) {
        print(e);
        EasyLoading.showError("Oops something went wrong");
      }
      return;
    }
  }

  doAuth({String tag = "", String? userId}) async {
    var partnerParams = null;
    if (currentUserId != null) {
      partnerParams = HashMap<String, String>();
      partnerParams["user_id"] = currentUserId;
    }
    var result = await SmileFlutter.captureSelfie(tag) ?? null;
    var resultCode = result!["SID_RESULT_CODE"];
    var resultTag = result["SID_RESULT_TAG"];
    if (resultCode == -1) {
      try {
        EasyLoading.show(status: 'loading...');
        var submitResult = await SmileFlutter.submitJob(resultTag, 2,
            isProduction, "https:test.com", partnerParams, null, null);
        EasyLoading.dismiss();
        processResponse(submitResult);
        return;
      } catch (e) {
        EasyLoading.showError("Oops something went wrong");
      }
      return;
    }
  }

  showIdInfoDialog(BuildContext context, int jobType, {String tag = ""}) {
    showDialog(
        context: context,
        builder: (BuildContext context) {
          return StatefulBuilder(builder: (context, setState) {
            return AlertDialog(
              scrollable: true,
              title: Text("Please enter id information"),
              content: Padding(
                padding: const EdgeInsets.all(8.0),
                child: Form(
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.stretch,
                    children: <Widget>[
                      DropdownButton<DropDownType>(
                        value: countryDropDownValue,
                        icon: const Icon(Icons.arrow_downward),
                        iconSize: 24,
                        elevation: 16,
                        style: const TextStyle(color: Colors.deepPurple),
                        underline: Container(
                          height: 2,
                          color: Colors.deepPurpleAccent,
                        ),
                        onChanged: (DropDownType? newValue) {
                          setState(() {
                            countryDropDownValue = newValue!;
                          });
                        },
                        items: <DropDownType>[
                          new DropDownType('Select Country', "select"),
                          new DropDownType('Ghana', "GH"),
                          new DropDownType('Kenya', "KE"),
                          new DropDownType('Nigeria', "NG"),
                          new DropDownType('South Africa', "ZA"),
                        ].map<DropdownMenuItem<DropDownType>>(
                            (DropDownType value) {
                          return DropdownMenuItem<DropDownType>(
                            value: value,
                            child: Text(value.name),
                          );
                        }).toList(),
                      ),
                      DropdownButton<DropDownType>(
                        value: idTypeDropDownValue,
                        icon: const Icon(Icons.arrow_downward),
                        iconSize: 24,
                        elevation: 16,
                        style: const TextStyle(color: Colors.deepPurple),
                        underline: Container(
                          height: 2,
                          color: Colors.deepPurpleAccent,
                        ),
                        onChanged: (DropDownType? newValue) {
                          setState(() {
                            idTypeDropDownValue = newValue!;
                            currentUserId = newValue.name;
                          });
                        },
                        items: <DropDownType>[
                          new DropDownType('Select Id Type', "select"),
                          new DropDownType(
                              "Driver's License", "DRIVERS_LICENSE"),
                          new DropDownType("Passport", "PASSPORT"),
                          new DropDownType("SSNIT", "SSNIT"),
                          new DropDownType("Voter ID", "VOTER_ID"),
                          new DropDownType("National ID", "NATIONAL_ID"),
                          new DropDownType("Alien Card", "ALIEN_CARD"),
                          new DropDownType("BVN", "BVN"),
                          new DropDownType("NIN", "NIN"),
                          new DropDownType("NIN SLIP", "NIN_SLIP"),
                          new DropDownType("TIN", "TIN"),
                          new DropDownType("CAC", "CAC"),
                        ].map<DropdownMenuItem<DropDownType>>(
                            (DropDownType value) {
                          return DropdownMenuItem<DropDownType>(
                            value: value,
                            child: Text(value.name),
                          );
                        }).toList(),
                      ),
                      TextFormField(
                        decoration: InputDecoration(labelText: "ID Number"),
                        controller: idNumberController,
                      ),
                      TextFormField(
                        decoration: InputDecoration(labelText: "First Name"),
                        controller: firstNameController,
                      ),
                      TextFormField(
                        decoration: InputDecoration(labelText: "Middle Name"),
                        controller: middleNameController,
                      ),
                      TextFormField(
                        decoration: InputDecoration(labelText: "Last Name"),
                        controller: lastNameController,
                      )
                    ],
                  ),
                ),
              ),
              actions: [
                Row(
                  mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                  children: [
                    OutlinedButton(
                        child: Text("Cancel"),
                        onPressed: () {
                          Navigator.of(context, rootNavigator: true).pop();
                          // your code
                        }),
                    OutlinedButton(
                        child: Text("Submit"),
                        onPressed: () async {
                          if (countryDropDownValue.value == "select") {
                            EasyLoading.showError("Please select country");
                            return;
                          }
                          if (idTypeDropDownValue.value == "select") {
                            EasyLoading.showError("Please select ID Type");
                            return;
                          }

                          if (idNumberController.text.isEmpty && jobType == 1) {
                            EasyLoading.showError("Please enter ID number");
                            return;
                          }
                          var userIdInfo = HashMap<String, String>();
                          if (countryDropDownValue != null) {
                            userIdInfo["country"] = countryDropDownValue.value;
                          }
                          if (idTypeDropDownValue != null) {
                            userIdInfo["id_type"] = idTypeDropDownValue.value;
                          }
                          if (!idNumberController.text.isEmpty) {
                            userIdInfo["id_number"] = idNumberController.text;
                          }

                          if (!firstNameController.text.isEmpty) {
                            userIdInfo["first_name"] = firstNameController.text;
                          }
                          if (!middleNameController.text.isEmpty) {
                            userIdInfo["middle_name"] =
                                middleNameController.text;
                          }
                          if (!lastNameController.text.isEmpty) {
                            userIdInfo["last_name"] = lastNameController.text;
                          }
                          Navigator.of(context, rootNavigator: true).pop();
                          try {
                            EasyLoading.show(status: 'loading...');
                            var submitResult = await SmileFlutter.submitJob(
                                tag,
                                jobType,
                                isProduction,
                                "https:test.com",
                                null,
                                userIdInfo,
                                null);
                            EasyLoading.dismiss();
                            processResponse(submitResult);
                            return;
                          } catch (e) {
                            countryDropDownValue =
                                new DropDownType('Select Country', "select");
                            idTypeDropDownValue =
                                new DropDownType('Select Id Type', "select");
                            idNumberController.text = "";
                            firstNameController.text = "";
                            middleNameController.text = "";
                            lastNameController.text = "";
                            EasyLoading.showError(jobType == 1
                                ? "Oops enroll with id number failed"
                                : "Oops document verification failed");
                          }
                          // your code
                        })
                  ],
                )
              ],
            );
          });
        });
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      resizeToAvoidBottomInset: false,
      appBar: AppBar(
        title: const Text('Smile ID Flutter Example'),
      ),
      body: Padding(
        padding: const EdgeInsets.all(10.0),
        child: Center(
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.stretch,
            children: [
              SwitchListTile(
                title: const Text('Run on Production'),
                value: isProduction,
                onChanged: (bool value) {
                  setState(() {
                    isProduction = value;
                  });
                },
                secondary: const Icon(Icons.lightbulb_outline),
              ),
              OutlinedButton(
                  onPressed: () async {
                    try {
                      var result = await SmileFlutter.captureSelfie("") ?? null;
                      handleSelfieResult(context, result);
                      // var result = await SmileFlutter.getCurrentTags() ?? null;
                      // print(result);
                    } catch (e) {
                      print(e);
                      EasyLoading.showError(
                          "Oops something went wrong " + e.toString());
                    }
                  },
                  child: Text("Selfie Test")),
              OutlinedButton(
                  onPressed: () async {
                    List<String> orientationList = [
                      "Device",
                      "Landscape",
                      "Portrait"
                    ];

                    SelectDialog.showModal<String>(
                      context,
                      label: "Select Orientation",
                      items: orientationList,
                      onChange: (String selected) {
                        setState(() async {
                          try {
                            var iDCaptureOrientation = 0;
                            if (selected == "Device") {
                              iDCaptureOrientation = 0;
                            } else if (selected == "Landscape") {
                              iDCaptureOrientation = 1;
                            } else if (selected == "Portrait") {
                              iDCaptureOrientation = 2;
                            }
                            var result = await SmileFlutter.captureIDCard("",
                                    iDCaptureOrientation:
                                        iDCaptureOrientation) ??
                                null;
                            // var result = await SmileFlutter.captureIDCard("MY_OWN_TAGS") ?? null;
                            handleSelfieResult(context, result);
                            // var result = await SmileFlutter.getCurrentTags() ?? null;
                            // print(result);
                            // handleSelfieResult(context, result);
                          } catch (e) {
                            print(e);
                            EasyLoading.showError(
                                "Oops something went wrong " + e.toString());
                          }
                        });
                      },
                    );
                  },
                  child: Text("ID Card Test")),
              OutlinedButton(
                  onPressed: () async {
                    try {
                      var result =
                          await SmileFlutter.captureSelfieAndIDCard("") ?? null;
                      handleSelfieResult(context, result);
                    } catch (e) {
                      print(e);
                      EasyLoading.showError(
                          "Oops something went wrong " + e.toString());
                    }
                  },
                  child: Text("Selfie and ID Card Test")),
              OutlinedButton(
                  onPressed: () async {
                    doEnroll();
                  },
                  child: Text("Enroll")),
              OutlinedButton(
                  onPressed: () {
                    doEnrollWithIDCard();
                  },
                  child: Text("Enroll with ID Card")),
              OutlinedButton(
                  onPressed: () {
                    doEnrollWithIDNumber(context);
                  },
                  child: Text("Enroll with ID Number")),
              OutlinedButton(
                  onPressed: () {
                    doDocVerificaton();
                  },
                  child: Text("Verify Doc")),
              OutlinedButton(
                  onPressed: () {
                    if (currentUserId != null) {
                      doAuth();
                    } else {
                      EasyLoading.showError(
                          "Please perform an enroll first before attempting to authenticate");
                    }
                  },
                  child: Text("Authenticate")),
              OutlinedButton(
                  onPressed: () async {
                    try {
                      var millis = 978296400000;
                      var dt = DateTime.fromMillisecondsSinceEpoch(millis);
                      var currentTag =
                          DateFormat('dd_MM_yyyy_HH_mm_ss').format(dt);
                      var result = await SmileFlutter.showConsent(
                              currentTag,
                              Platform.isAndroid ? "app_icon" : "AppIcon",
                              "com.smileidentity.smileFlutterExample",
                              "Smile ID Test",
                              "https://docs.smileidentity.com/") ??
                          null;
                      var resultCode = result!["SID_RESULT_CODE"];
                      if (resultCode == 1) {
                        doEnroll(tag: currentTag);
                        return;
                      }
                      EasyLoading.showError("User did not allow consent");
                    } catch (e) {
                      EasyLoading.showError("Oops something went wrong $e");
                    }
                  },
                  child: Text("Show Consent")),
            ],
          ),
        ),
      ),
    );
  }
}
